@@ -57,6 +57,19 @@ static void LIBPQ_update_transaction_status(ConnectionClass *self);
 extern GLOBAL_VALUES globals;
 
 
+static void CC_set_error_if_not_set(ConnectionClass *self, int errornumber, const char *errormsg, const char *func)
+{
+	if (CC_get_errornumber(self) <= 0)
+	{
+		if (CC_get_errormsg(self) == NULL)
+			CC_set_error(self, errornumber, errormsg, func);
+		else
+			CC_set_errornumber(self, errornumber);
+	}
+	else
+		CC_set_errormsg(self, errormsg);
+}
+
 RETCODE		SQL_API
 PGAPI_AllocConnect(HENV henv,
 				   HDBC * phdbc)
@@ -252,102 +265,6 @@ PGAPI_FreeConnect(HDBC hdbc)
 	return SQL_SUCCESS;
 }
 
-static void
-CC_conninfo_release(ConnInfo *conninfo)
-{
-	NULL_THE_NAME(conninfo->password);
-	NULL_THE_NAME(conninfo->conn_settings);
-	finalize_globals(&conninfo->drivers);
-}
-
-void
-CC_conninfo_init(ConnInfo *conninfo, UInt4 option)
-{
-	CSTR	func = "CC_conninfo_init";
-	mylog("%s opt=%d\n", func, option);
-
-	if (0 != (CLEANUP_FOR_REUSE & option))
-		CC_conninfo_release(conninfo);
-	memset(conninfo, 0, sizeof(ConnInfo));
-	conninfo->allow_keyset = -1;
-	conninfo->lf_conversion = -1;
-	conninfo->true_is_minus1 = -1;
-	conninfo->int8_as = -101;
-	conninfo->bytea_as_longvarbinary = -1;
-	conninfo->use_server_side_prepare = -1;
-	conninfo->lower_case_identifier = -1;
-	conninfo->rollback_on_error = -1;
-	conninfo->force_abbrev_connstr = -1;
-	conninfo->bde_environment = -1;
-	conninfo->fake_mss = -1;
-	conninfo->cvt_null_date_string = -1;
-	conninfo->accessible_only = -1;
-	conninfo->ignore_round_trip_time = -1;
-	conninfo->disable_keepalive = -1;
-	conninfo->gssauth_use_gssapi = -1;
-	conninfo->keepalive_idle = -1;
-	conninfo->keepalive_interval = -1;
-#ifdef	_HANDLE_ENLIST_IN_DTC_
-	conninfo->xa_opt = -1;
-#endif /* _HANDLE_ENLIST_IN_DTC_ */
-	if (0 != (COPY_GLOBALS & option))
-		copy_globals(&(conninfo->drivers), &globals);
-}
-
-#define	CORR_STRCPY(item)	strncpy_null(ci->item, sci->item, sizeof(ci->item))
-#define	CORR_VALCPY(item)	(ci->item = sci->item)
-
-void
-CC_copy_conninfo(ConnInfo *ci, const ConnInfo *sci)
-{
-	memset(ci, 0,sizeof(ConnInfo));
-
-	CORR_STRCPY(dsn);
-	CORR_STRCPY(desc);
-	CORR_STRCPY(drivername);
-	CORR_STRCPY(server);
-	CORR_STRCPY(database);
-	CORR_STRCPY(username);
-	NAME_TO_NAME(ci->password, sci->password);
-	CORR_STRCPY(port);
-	CORR_STRCPY(sslmode);
-	CORR_STRCPY(onlyread);
-	CORR_STRCPY(fake_oid_index);
-	CORR_STRCPY(show_oid_column);
-	CORR_STRCPY(row_versioning);
-	CORR_STRCPY(show_system_tables);
-	CORR_STRCPY(translation_dll);
-	CORR_STRCPY(translation_option);
-	CORR_VALCPY(password_required);
-	NAME_TO_NAME(ci->conn_settings, sci->conn_settings);
-	CORR_VALCPY(allow_keyset);
-	CORR_VALCPY(updatable_cursors);
-	CORR_VALCPY(lf_conversion);
-	CORR_VALCPY(true_is_minus1);
-	CORR_VALCPY(int8_as);
-	CORR_VALCPY(bytea_as_longvarbinary);
-	CORR_VALCPY(use_server_side_prepare);
-	CORR_VALCPY(lower_case_identifier);
-	CORR_VALCPY(rollback_on_error);
-	CORR_VALCPY(force_abbrev_connstr);
-	CORR_VALCPY(bde_environment);
-	CORR_VALCPY(fake_mss);
-	CORR_VALCPY(cvt_null_date_string);
-	CORR_VALCPY(accessible_only);
-	CORR_VALCPY(ignore_round_trip_time);
-	CORR_VALCPY(disable_keepalive);
-	CORR_VALCPY(gssauth_use_gssapi);
-	CORR_VALCPY(extra_opts);
-	CORR_VALCPY(keepalive_idle);
-	CORR_VALCPY(keepalive_interval);
-#ifdef	_HANDLE_ENLIST_IN_DTC_
-	CORR_VALCPY(xa_opt);
-#endif
-	copy_globals(&(ci->drivers), &(sci->drivers));	/* moved from driver's option */
-}
-#undef	CORR_STRCPY
-#undef	CORR_VALCPY
-
 /*
  *		IMPLEMENTATION CONNECTION CLASS
  */
@@ -408,7 +325,7 @@ CC_initialize(ConnectionClass *rv, BOOL lockinit)
 	rv->lobj_type = PG_TYPE_LO_UNDEFINED;
 	if (isMsAccess())
 		rv->ms_jet = 1;
-	rv->isolation = SQL_TXN_READ_COMMITTED;
+	rv->isolation = 0; // means initially unknown server's default isolation
 	rv->mb_maxbyte_per_char = 1;
 	rv->max_identifier_length = -1;
 	rv->autocommit_public = SQL_AUTOCOMMIT_ON;
@@ -801,13 +718,17 @@ CC_set_translation(ConnectionClass *self)
 	return TRUE;
 }
 
+#ifndef PG_DIAG_SEVERITY_NONLOCALIZED
+#define PG_DIAG_SEVERITY_NONLOCALIZED 'V'
+#endif
+
 void
 handle_pgres_error(ConnectionClass *self, const PGresult *pgres,
 				   const char *comment,
-				   QResultClass *res, BOOL fatal)
+				   QResultClass *res, BOOL error_not_a_notice)
 {
-	UDWORD		abort_opt;
 	char	   *errseverity;
+	char	   *errseverity_nonloc = NULL;
 	char	   *errprimary;
 	char	   *errmsg = NULL;
 	size_t		errmsglen;
@@ -828,6 +749,11 @@ handle_pgres_error(ConnectionClass *self, const PGresult *pgres,
 	 * severity and primary message, which is backwards compatible.
 	 */
 	errseverity = PQresultErrorField(pgres, PG_DIAG_SEVERITY);
+	if (PG_VERSION_GE(self, 9.6))
+	{
+		errseverity_nonloc = PQresultErrorField(pgres, PG_DIAG_SEVERITY_NONLOCALIZED);
+		mylog("PG_DIAG_SEVERITY_NONLOCALIZED=%s\n", errseverity_nonloc ? errseverity_nonloc : "(null)");
+	}
 	errprimary = PQresultErrorField(pgres, PG_DIAG_MESSAGE_PRIMARY);
 	if (errprimary == NULL)
 	{
@@ -848,51 +774,52 @@ handle_pgres_error(ConnectionClass *self, const PGresult *pgres,
 	if (errmsg == NULL)
 		errmsg = errprimary;
 
-	abort_opt = 0;
-
-	if (PQstatus(self->pqconn) == CONNECTION_BAD)
-	{
-		CC_set_errornumber(self, CONNECTION_SERVER_REPORTED_ERROR);
-		abort_opt = CONN_DEAD;
-	}
-	else
-	{
-		CC_set_errornumber(self, CONNECTION_SERVER_REPORTED_WARNING);
-		if (fatal && CC_is_in_trans(self))
-			CC_set_in_error_trans(self);
-	}
-
-	mylog("notice/error message len=%d\n", strlen(errmsg));
-
-	if (fatal)
-	{
-		if (res)
-		{
-			QR_set_rstatus(res, PORES_FATAL_ERROR);
-			if (errmsg[0])
-				QR_set_message(res, errmsg);
-			QR_set_aborted(res, TRUE);
-		}
-	}
-	else
+	if (!error_not_a_notice) /* warning, notice, log etc */
 	{
+		mylog("notice message %s\n", errmsg);
 		if (res)
 		{
 			if (QR_command_successful(res))
-				QR_set_rstatus(res, PORES_NONFATAL_ERROR);
+				QR_set_rstatus(res, PORES_NONFATAL_ERROR); /* notice or warning */
 			QR_set_notice(res, errmsg);  /* will dup this string */
 		}
+		goto cleanup;
 	}
+
+	mylog("error message len=%d\n", strlen(errmsg));
+
+	if (res)
+	{
+		QR_set_rstatus(res, PORES_FATAL_ERROR); /* error or fatal */
+		if (errmsg[0])
+			QR_set_message(res, errmsg);
+		QR_set_aborted(res, TRUE);
+	}
+
+	/*
+	 *	If the error is continuable after rollback?
+	 */
+	if (PQstatus(self->pqconn) == CONNECTION_BAD)
+	{
+		CC_set_errornumber(self, CONNECTION_COMMUNICATION_ERROR);
+		CC_on_abort(self, CONN_DEAD); /* give up the connection */
+	}
+	else if ((errseverity_nonloc && strcmp(errseverity_nonloc, "FATAL") == 0) ||
+		(NULL == errseverity_nonloc && errseverity && strcmp(errseverity, "FATAL") == 0)) /* no */ 
+	{
+		CC_set_errornumber(self, CONNECTION_SERVER_REPORTED_SEVERITY_FATAL);
+		CC_on_abort(self, CONN_DEAD); /* give up the connection */
+	}
+	else /* yes */
+	{
+		CC_set_errornumber(self, CONNECTION_SERVER_REPORTED_SEVERITY_ERROR);
+		if (CC_is_in_trans(self))
+			CC_set_in_error_trans(self);
+	}
+
+cleanup:
 	if (errmsg != errprimary)
 		free(errmsg);
-
-	if (0 != abort_opt
-#ifdef	_LEGACY_MODE_
-		|| TRUE
-#endif /* _LEGACY_NODE_ */
-		)
-		CC_on_abort(self, abort_opt);
-
 	LIBPQ_update_transaction_status(self);
 }
 
@@ -977,9 +904,14 @@ static char CC_initial_log(ConnectionClass *self, const char *func)
 		 ci->drivers.extra_systable_prefixes,
 		 PRINT_NAME(ci->drivers.conn_settings),
 		 encoding ? encoding : "");
-	if (self->status != CONN_NOT_CONNECTED)
+	if (self->status == CONN_DOWN)
 	{
-		CC_set_error(self, CONN_OPENDB_ERROR, "Already connected.", func);
+		CC_set_error_if_not_set(self, CONN_OPENDB_ERROR, "Connection broken.", func);
+		return 0;
+	}
+	else if (self->status != CONN_NOT_CONNECTED)
+	{
+		CC_set_error_if_not_set(self, CONN_OPENDB_ERROR, "Already connected.", func);
 		return 0;
 	}
 
@@ -988,12 +920,17 @@ static char CC_initial_log(ConnectionClass *self, const char *func)
 	return 1;
 }
 
+static int handle_show_results(const QResultClass *res);
+#define	TRANSACTION_ISOLATION "transaction_isolation"
+#define	ISOLATION_SHOW_QUERY "show " TRANSACTION_ISOLATION
+
 static int LIBPQ_connect(ConnectionClass *self);
 static char
 LIBPQ_CC_connect(ConnectionClass *self, char *salt_para)
 {
 	int		ret;
 	CSTR		func = "LIBPQ_CC_connect";
+	QResultClass	*res;
 
 	mylog("%s: entering...\n", func);
 
@@ -1002,9 +939,17 @@ LIBPQ_CC_connect(ConnectionClass *self, char *salt_para)
 
 	if (ret = LIBPQ_connect(self), ret <= 0)
 		return ret;
-	CC_send_settings(self, "SET DateStyle = 'ISO';SET extra_float_digits = 2");
+	res = CC_send_query(self, "SET DateStyle = 'ISO';SET extra_float_digits = 2;" ISOLATION_SHOW_QUERY, NULL, 0, NULL);
+	if (QR_command_maybe_successful(res))
+	{
+		handle_show_results(res);
+		ret = 1;
+	}
+	else
+		ret = 0;
+	QR_Destructor(res);
 
-	return 1;
+	return ret;
 }
 
 char
@@ -1036,10 +981,10 @@ CC_connect(ConnectionClass *self, char *salt_para)
 	 */
 
 	/* Global settings */
- retsend = CC_send_settings(self, GET_NAME(self->connInfo.drivers.conn_settings));
+	retsend = CC_send_settings(self, GET_NAME(self->connInfo.drivers.conn_settings));
 	/* Per Datasource settings */
- if (retsend)
-	 retsend = CC_send_settings(self, GET_NAME(self->connInfo.conn_settings));
+	if (retsend)
+		retsend = CC_send_settings(self, GET_NAME(self->connInfo.conn_settings));
 
 	if (CC_get_errornumber(self) > 0)
 		saverr = strdup(CC_get_errormsg(self));
@@ -1067,13 +1012,20 @@ CC_connect(ConnectionClass *self, char *salt_para)
 	}
 #endif /* UNICODE_SUPPORT */
 
+	if (self->server_isolation != self->isolation)
+		if (!CC_set_transact(self, self->isolation))
+		{
+			ret = 0;
+			goto cleanup;
+		}
+
 	ci->updatable_cursors = DISALLOW_UPDATABLE_CURSORS;
 	if (ci->allow_keyset)
 	{
 		if (ci->drivers.lie || !ci->drivers.use_declarefetch)
 			ci->updatable_cursors |= (ALLOW_STATIC_CURSORS | ALLOW_KEYSET_DRIVEN_CURSORS | ALLOW_BULK_OPERATIONS | SENSE_SELF_OPERATIONS);
 		else
-			ci->updatable_cursors |= (ALLOW_STATIC_CURSORS | SENSE_SELF_OPERATIONS);
+			ci->updatable_cursors |= (ALLOW_STATIC_CURSORS | ALLOW_BULK_OPERATIONS | SENSE_SELF_OPERATIONS);
 	}
 
 	if (CC_get_errornumber(self) > 0)
@@ -1219,6 +1171,67 @@ mylog("max_identifier_length=%d\n", len);
 	return len < 0 ? 0 : len;
 }
 
+static SQLINTEGER
+isolation_str_to_enum(const char *str_isolation)
+{
+	SQLINTEGER	isolation = 0;
+
+	if (strnicmp(str_isolation, "seri", 4) == 0)
+		isolation = SQL_TXN_SERIALIZABLE;
+	else if (strnicmp(str_isolation, "repe", 4) == 0)
+		isolation = SQL_TXN_REPEATABLE_READ;
+	else if (strnicmp(str_isolation, "read com", 8) == 0)
+		isolation = SQL_TXN_READ_COMMITTED;
+	else if (strnicmp(str_isolation, "read unc", 8) == 0)
+		isolation = SQL_TXN_READ_UNCOMMITTED;
+
+	return isolation;
+}
+
+static int handle_show_results(const QResultClass *res)
+{
+	int			count = 0;
+	const QResultClass	*qres;
+	ConnectionClass		*conn = QR_get_conn(res);
+
+	for (qres = res; qres; qres = qres->next)
+	{
+		if (!qres->command ||
+		    stricmp(qres->command, "SHOW") != 0)
+			continue;
+		if (strcmp(QR_get_fieldname(qres, 0), TRANSACTION_ISOLATION) == 0)
+		{
+			conn->server_isolation = isolation_str_to_enum(QR_get_value_backend_text(qres, 0, 0));
+			mylog("isolation %d to be %d\n", conn->server_isolation, conn->isolation);
+			if (0 == conn->isolation)
+				conn->isolation = conn->server_isolation;
+			if (0 == conn->default_isolation)
+				conn->default_isolation = conn->server_isolation;
+			count++;
+		}
+	}
+
+	return count;
+}
+/*
+ *	This function may not be called as long as ISOLATION_SHOW_QUERY is
+ *	issued in LIBPQ_CC_connect.
+ */
+SQLUINTEGER	CC_get_isolation(ConnectionClass *self)
+{
+	SQLUINTEGER	isolation = 0;
+	QResultClass	*res;
+
+	res = CC_send_query(self, ISOLATION_SHOW_QUERY, NULL, ROLLBACK_ON_ERROR | IGNORE_ABORT_ON_CONN, NULL);
+	if (QR_command_maybe_successful(res))
+	{
+		handle_show_results(res);
+		isolation = self->server_isolation;
+	}
+	QR_Destructor(res);
+mylog("isolation=%d\n", isolation);
+	return isolation;
+}
 
 void
 CC_set_error(ConnectionClass *self, int number, const char *message, const char *func)
@@ -1687,7 +1700,7 @@ CC_send_query_append(ConnectionClass *self, const char *query, QueryInfo *qi, UD
 	if (!PQsendQuery(self->pqconn, query_buf))
 	{
 		char *errmsg = PQerrorMessage(self->pqconn);
-		CC_set_error(self, CONNECTION_SERVER_NOT_REACHED, errmsg, func);
+		CC_set_error(self, CONNECTION_COMMUNICATION_ERROR, errmsg, func);
 		goto cleanup;
 	}
 	PQsetSingleRowMode(self->pqconn);
@@ -2026,6 +2039,8 @@ cleanup:
 	 */
 	LIBPQ_update_transaction_status(self);
 
+	if (retres)
+		QR_set_conn(retres, self);
 	return retres;
 }
 
@@ -2038,6 +2053,37 @@ static const char *func_param_str[MAX_SEND_FUNC_ARGS + 1] =
 	"($1, $2, $3)"
 };
 
+static
+Int8 odbc_hton64(Int8 h64)
+{
+	union {
+		Int8	n64;
+		UInt4	i32[2];
+	} u;
+
+	u.i32[0] = htonl((UInt4) (h64 >> 32));
+	u.i32[1] = htonl((UInt4) h64);
+
+	return u.n64;
+}
+
+static
+Int8 odbc_ntoh64(Int8 n64)
+{
+	union {
+		Int8	h64;
+		UInt4	i32[2];
+	} u;
+	Int8 result;
+
+	u.h64 = n64;
+	result = ntohl(u.i32[0]);
+	result <<= 32;
+	result |= ntohl(u.i32[1]);
+
+	return result;
+}
+
 
 int
 CC_send_function(ConnectionClass *self, const char *fn_name, void *result_buf, int *actual_result_len, int result_is_int, LO_ARG *args, int nargs)
@@ -2052,6 +2098,7 @@ CC_send_function(ConnectionClass *self, const char *fn_name, void *result_buf, i
 	int			paramLengths[MAX_SEND_FUNC_ARGS];
 	int			paramFormats[MAX_SEND_FUNC_ARGS];
 	Int4		intParamBufs[MAX_SEND_FUNC_ARGS];
+	Int8		int8ParamBufs[MAX_SEND_FUNC_ARGS];
 
 	mylog("send_function(): conn=%p, fn_name=%s, result_is_int=%d, nargs=%d\n", self, fn_name, result_is_int, nargs);
 
@@ -2063,10 +2110,17 @@ CC_send_function(ConnectionClass *self, const char *fn_name, void *result_buf, i
 			 func_param_str[nargs]);
 	for (i = 0; i < nargs; ++i)
 	{
-		mylog("  arg[%d]: len = %d, isint = %d, integer = %d, ptr = %p\n", i, args[i].len, args[i].isint, args[i].u.integer, args[i].u.ptr);
-
+		mylog("  arg[%d]: len = %d, isint = %d, integer = " FORMATI64 ", ptr = %p\n", i, args[i].len, args[i].isint, args[i].isint == 2 ? args[i].u.integer64 : args[i].u.integer, args[i].u.ptr);
 		/* integers are sent as binary, others as text */
-		if (args[i].isint)
+		if (args[i].isint == 2)
+		{
+			paramTypes[i] = PG_TYPE_INT8;
+			int8ParamBufs[i] = odbc_hton64(args[i].u.integer64);
+			paramValues[i] = (char *) &int8ParamBufs[i];
+			paramLengths[i] = 8;
+			paramFormats[i] = 1;
+		}
+		else if (args[i].isint)
 		{
 			paramTypes[i] = PG_TYPE_INT4;
 			intParamBufs[i] = htonl(args[i].u.integer);
@@ -2108,7 +2162,15 @@ CC_send_function(ConnectionClass *self, const char *fn_name, void *result_buf, i
 	if (*actual_result_len > 0)
 	{
 		char *value = PQgetvalue(pgres, 0, 0);
-		if (result_is_int)
+		if (result_is_int == 2)
+		{
+			Int8 int8val;
+			memcpy(&int8val, value, sizeof(Int8));
+			int8val = odbc_ntoh64(int8val);
+			memcpy(result_buf, &int8val, sizeof(Int8));
+mylog("int8 result=" FORMATI64 "\n", int8val);
+		}
+		else if (result_is_int)
 		{
 			Int4 int4val;
 			memcpy(&int4val, value, sizeof(Int4));
@@ -2374,7 +2436,7 @@ LIBPQ_update_transaction_status(ConnectionClass *self)
 	}
 }
 
-#define        PROTOCOL3_OPTS_MAX      20
+#define        PROTOCOL3_OPTS_MAX      30
 
 static int
 LIBPQ_connect(ConnectionClass *self)
@@ -2386,13 +2448,26 @@ LIBPQ_connect(ConnectionClass *self)
 	int			pqret;
 	int			pversion;
 	const	char	*opts[PROTOCOL3_OPTS_MAX], *vals[PROTOCOL3_OPTS_MAX];
-	int			cnt;
+	PQconninfoOption	*conninfoOption = NULL, *pqopt;
+	int			i, cnt;
 	char		login_timeout_str[20];
 	char		keepalive_idle_str[20];
 	char		keepalive_interval_str[20];
+	char		*errmsg = NULL;
 
-	mylog("connecting to the database using %s as the server\n", self->connInfo.server);
+	mylog("connecting to the database using %s as the server and pqopt={%s}\n", self->connInfo.server, SAFE_NAME(ci->pqopt));
 
+	if (NULL == (conninfoOption = PQconninfoParse(SAFE_NAME(ci->pqopt), &errmsg)))
+	{
+		char emsg[200];
+
+		if (errmsg != NULL)
+			snprintf(emsg, sizeof(emsg), "libpq connection parameter error:%s", errmsg);
+		else
+			strncpy_null(emsg, "memory error? in PQconninfoParse", sizeof(emsg));
+		CC_set_error(self, CONN_OPENDB_ERROR, emsg, func);
+		goto cleanup;
+	}
 	/* Build arrays of keywords & values, for PQconnectDBParams */
 	cnt = 0;
 	if (ci->server[0])
@@ -2460,8 +2535,36 @@ LIBPQ_connect(ConnectionClass *self)
 		sprintf(keepalive_interval_str, "%d", self->connInfo.keepalive_interval);
 		opts[cnt] = "keepalives_interval";	vals[cnt++] = keepalive_interval_str;
 	}
-	opts[cnt] = vals[cnt] = NULL;
+	if (conninfoOption != NULL)
+	{
+		const char *keyword, *val;
+		int j;
 
+		for (i = 0, pqopt = conninfoOption; (keyword = pqopt->keyword) != NULL; i++, pqopt++)
+		{
+			if ((val = pqopt->val) != NULL)
+			{
+				for (j = 0; j < cnt; j++)
+				{
+					if (stricmp(opts[j], keyword) == 0)
+					{
+						char emsg[100];
+
+						if (vals[j] != NULL && strcmp(vals[j], val) == 0)
+							continue;
+						snprintf(emsg, sizeof(emsg), "%s parameter in pqopt option conflicts with other ordinary option", keyword);
+						CC_set_error(self, CONN_OPENDB_ERROR, emsg, func);
+						goto cleanup;
+					}
+				}
+				if (j >= cnt && cnt < PROTOCOL3_OPTS_MAX - 1)
+				{
+					opts[cnt] = keyword; vals[cnt++] = val;
+				}
+			}
+		}
+	}
+	opts[cnt] = vals[cnt] = NULL;
 	/* Ok, we're all set to connect */
 
 	pqconn = PQconnectdbParams(opts, vals, FALSE);
@@ -2522,6 +2625,9 @@ inolog("status=%d\n", pqret);
 	ret = 1;
 
 cleanup:
+	if (errmsg != NULL)
+		free(errmsg);
+	PQconninfoFree(conninfoOption);
 	if (ret != 1)
 	{
 		if (self->pqconn)
@@ -2576,6 +2682,127 @@ const char *CurrCatString(const ConnectionClass *conn)
 	return cat;
 }
 
+/*------
+ *	Create a null terminated lower-case string if the
+ *	original string contains upper-case characters.
+ *	The SQL_NTS length is considered.
+ *------
+ */
+SQLCHAR *
+make_lstring_ifneeded(ConnectionClass *conn, const SQLCHAR *s, ssize_t len, BOOL ifallupper)
+{
+	ssize_t	length = len;
+	char	   *str = NULL;
+	const char *ccs = (const char *) s;
+
+	if (s && (len > 0 || (len == SQL_NTS && (length = strlen(ccs)) > 0)))
+	{
+		int	i;
+		const UCHAR *ptr;
+		encoded_str encstr;
+
+		make_encoded_str(&encstr, conn, ccs);
+		for (i = 0, ptr = (const UCHAR *) ccs; i < length; i++, ptr++)
+		{
+			encoded_nextchar(&encstr);
+			if (ENCODE_STATUS(encstr) != 0)
+				continue;
+			if (ifallupper && islower(*ptr))
+			{
+				if (str)
+				{
+					free(str);
+					str = NULL;
+				}
+				break;
+			}
+			if (tolower(*ptr) != *ptr)
+			{
+				if (!str)
+				{
+					str = malloc(length + 1);
+					if (!str) return NULL;
+					memcpy(str, s, length);
+					str[length] = '\0';
+				}
+				str[i] = tolower(*ptr);
+			}
+		}
+	}
+
+	return (SQLCHAR *) str;
+}
+
+/*
+ *	Concatenate a single formatted argument to a given buffer handling the SQL_NTS thing.
+ *	"fmt" must contain somewhere in it the single form '%.*s'.
+ *	This is heavily used in creating queries for info routines (SQLTables, SQLColumns).
+ *	This routine could be modified to use vsprintf() to handle multiple arguments.
+ */
+static char *
+my_strcat(char *buf, const char *fmt, const char *s, ssize_t len)
+{
+	if (s && (len > 0 || (len == SQL_NTS && strlen(s) > 0)))
+	{
+		size_t			length = (len > 0) ? len : strlen(s);
+		size_t			pos = strlen(buf);
+
+		sprintf(&buf[pos], fmt, length, s);
+		return buf;
+	}
+	return NULL;
+}
+
+/*
+ *	my_strcat1 is a extension of my_strcat.
+ *	It can have 1 more parameter than my_strcat.
+ */
+static char *
+my_strcat1(char *buf, const char *fmt, const char *s1, const char *s)
+{
+	if (s && s[0] != '\0')
+	{
+		size_t	pos = strlen(buf);
+		ssize_t	length = strlen(s);
+
+		if (s1)
+			sprintf(&buf[pos], fmt, s1, length, s);
+		else
+			sprintf(&buf[pos], fmt, length, s);
+		return buf;
+	}
+	return NULL;
+}
+
+char *
+schema_strcat(char *buf, const char *fmt, const SQLCHAR *s, SQLLEN len, const SQLCHAR *tbname, SQLLEN tbnmlen, ConnectionClass *conn)
+{
+	if (!s || 0 == len)
+	{
+		/*
+		 * Note that this driver assumes the implicit schema is
+		 * the CURRENT_SCHEMA() though it doesn't worth the
+		 * naming.
+		 */
+		if (tbname && (tbnmlen > 0 || tbnmlen == SQL_NTS))
+			return my_strcat(buf, fmt, CC_get_current_schema(conn), SQL_NTS);
+		return NULL;
+	}
+	return my_strcat(buf, fmt, (char *) s, len);
+}
+
+char *
+schema_strcat1(char *buf, const char *fmt, const char *s1, const char *s, const SQLCHAR *tbname, int tbnmlen, ConnectionClass *conn)
+{
+	if (!s || s[0] == '\0')
+	{
+		if (tbname && (tbnmlen > 0 || tbnmlen == SQL_NTS))
+			return my_strcat1(buf, fmt, s1, CC_get_current_schema(conn));
+		return NULL;
+	}
+	return my_strcat1(buf, fmt, s1, s);
+}
+
 #ifdef	_HANDLE_ENLIST_IN_DTC_
 	/*
 	 *	Export the following functions so that the pgenlist dll
@@ -2982,3 +3209,54 @@ PgDtc_isolate(void *self, DWORD option)
 }
 
 #endif /* _HANDLE_ENLIST_IN_DTC_ */
+
+BOOL
+CC_set_transact(ConnectionClass *self, UInt4 isolation)
+{
+	char *query;
+	QResultClass *res;
+	BOOL	bShow = FALSE;
+
+	if (PG_VERSION_LT(self, 8.0) &&
+		(isolation == SQL_TXN_READ_UNCOMMITTED ||
+		 isolation == SQL_TXN_REPEATABLE_READ))
+	{
+		CC_set_error(self, CONN_NOT_IMPLEMENTED_ERROR, "READ_UNCOMMITTED or REPEATABLE_READ is not supported by the server", __FUNCTION__);
+		return FALSE;
+	}
+
+	switch (isolation)
+	{
+		case SQL_TXN_SERIALIZABLE:
+			query = "SET SESSION CHARACTERISTICS AS TRANSACTION ISOLATION LEVEL SERIALIZABLE";
+						break;
+		case SQL_TXN_REPEATABLE_READ:
+			query = "SET SESSION CHARACTERISTICS AS TRANSACTION ISOLATION LEVEL REPEATABLE READ";
+			break;
+		case SQL_TXN_READ_UNCOMMITTED:
+			query = "SET SESSION CHARACTERISTICS AS TRANSACTION ISOLATION LEVEL READ UNCOMMITTED";
+			break;
+		default:
+			query = "SET SESSION CHARACTERISTICS AS TRANSACTION ISOLATION LEVEL READ COMMITTED";
+			break;
+	}
+	if (self->default_isolation == 0)
+		bShow = TRUE;
+	if (bShow)
+		res = CC_send_query_append(self, ISOLATION_SHOW_QUERY, NULL, 0, NULL, query);
+	else
+		res = CC_send_query(self, query, NULL, 0, NULL);
+	if (!QR_command_maybe_successful(res))
+	{
+		CC_set_error(self, CONN_EXEC_ERROR, "ISOLATION change request to the server error", __FUNCTION__);
+		QR_Destructor(res);
+		return FALSE;
+	}
+	if (bShow)
+		handle_show_results(res);
+	QR_Destructor(res);
+	self->server_isolation = isolation;
+
+	return TRUE;
+}
+
