@@ -51,9 +51,9 @@ enum
 #define CONNECTION_NO_SUCH_DATABASE				105
 #define CONNECTION_BACKEND_CRAZY				106
 #define CONNECTION_NO_RESPONSE					107
-#define CONNECTION_SERVER_REPORTED_ERROR			108
+#define CONNECTION_SERVER_REPORTED_SEVERITY_FATAL		108
 #define CONNECTION_COULD_NOT_RECEIVE				109
-#define CONNECTION_SERVER_REPORTED_WARNING			110
+#define CONNECTION_SERVER_REPORTED_SEVERITY_ERROR		110
 #define CONNECTION_NEED_PASSWORD				112
 #define CONNECTION_COMMUNICATION_ERROR				113
 
@@ -74,7 +74,7 @@ enum
 #define CONN_INVALID_AUTHENTICATION					210
 #define CONN_AUTH_TYPE_UNSUPPORTED					211
 #define CONN_UNABLE_TO_LOAD_DLL						212
-
+#define CONN_ILLEGAL_TRANSACT_STATE					213
 #define CONN_VALUE_OUT_OF_RANGE						214
 
 #define CONN_OPTION_NOT_FOR_THE_DRIVER					216
@@ -86,6 +86,9 @@ enum
 #define CONN_IN_MANUAL_TRANSACTION	(1L<<2)
 #define CONN_IN_ERROR_BEFORE_IDLE	(1L<<3)
 
+/* not connected yet || already disconnected */
+#define	CC_not_connected(x)	(CONN_DOWN == (x)->status || CONN_NOT_CONNECTED == (x)->status)
+
 /* AutoCommit functions */
 #define CC_is_in_autocommit(x)		(x->transact_status & CONN_IN_AUTOCOMMIT)
 #define CC_does_autocommit(x) (CONN_IN_AUTOCOMMIT == ((x)->transact_status & (CONN_IN_AUTOCOMMIT | CONN_IN_MANUAL_TRANSACTION)))
@@ -204,57 +207,6 @@ do { \
 	} \
 } while (0)
 
-/*	Structure to hold all the connection attributes for a specific
-	connection (used for both registry and file, DSN and DRIVER)
-*/
-typedef struct
-{
-	char		dsn[MEDIUM_REGISTRY_LEN];
-	char		desc[MEDIUM_REGISTRY_LEN];
-	char		drivername[MEDIUM_REGISTRY_LEN];
-	char		server[MEDIUM_REGISTRY_LEN];
-	char		database[MEDIUM_REGISTRY_LEN];
-	char		username[MEDIUM_REGISTRY_LEN];
-	pgNAME		password;
-	char		port[SMALL_REGISTRY_LEN];
-	char		sslmode[16];
-	char		onlyread[SMALL_REGISTRY_LEN];
-	char		fake_oid_index[SMALL_REGISTRY_LEN];
-	char		show_oid_column[SMALL_REGISTRY_LEN];
-	char		row_versioning[SMALL_REGISTRY_LEN];
-	char		show_system_tables[SMALL_REGISTRY_LEN];
-	char		translation_dll[MEDIUM_REGISTRY_LEN];
-	char		translation_option[SMALL_REGISTRY_LEN];
-	char		password_required;
-	pgNAME		conn_settings;
-	signed char	allow_keyset;
-	signed char	updatable_cursors;
-	signed char	lf_conversion;
-	signed char	true_is_minus1;
-	signed char	int8_as;
-	signed char	bytea_as_longvarbinary;
-	signed char	use_server_side_prepare;
-	signed char	lower_case_identifier;
-	signed char	rollback_on_error;
-	signed char	force_abbrev_connstr;
-	signed char	bde_environment;
-	signed char	fake_mss;
-	signed char	cvt_null_date_string;
-	signed char	accessible_only;
-	signed char	ignore_round_trip_time;
-	signed char	disable_keepalive;
-	signed char	gssauth_use_gssapi;
-	UInt4		extra_opts;
-	Int4		keepalive_idle;
-	Int4		keepalive_interval;
-#ifdef	_HANDLE_ENLIST_IN_DTC_
-	signed char	xa_opt;
-#endif /* _HANDLE_ENLIST_IN_DTC_ */
-	GLOBAL_VALUES drivers;		/* moved from driver's option */
-} ConnInfo;
-
-#define SUPPORT_DESCRIBE_PARAM(conninfo_) (conninfo_->use_server_side_prepare)
-
 /*
  *	Macros to compare the server's version with a specified version
  *		1st parameter: pointer to a ConnectionClass object
@@ -367,7 +319,8 @@ struct ConnectionClass_
 	char		*server_encoding;
 	Int2		ccsc;
 	Int2		mb_maxbyte_per_char;
-	UInt4		isolation;
+	SQLUINTEGER	isolation;		/* isolation level initially unknown */
+	SQLUINTEGER	server_isolation;	/* isolation at server initially unknown */
 	char		*current_schema;
 	char		current_schema_valid;	/* is current_schema valid? TRUE when
 										 * current_schema == NULL means it's
@@ -378,6 +331,7 @@ struct ConnectionClass_
 	Int2		num_discardp;
 	char		**discardp;
 	int		num_descs;
+	SQLUINTEGER	default_isolation;	/* server's default isolation initially unkown */
 	DescriptorClass	**descs;
 	pgNAME		schemaIns;
 	pgNAME		tableIns;
@@ -406,9 +360,6 @@ struct ConnectionClass_
 #define CC_fake_mss(x)	(/* 0 != (x)->ms_jet && */ 0 < (x)->connInfo.fake_mss)
 #define CC_accessible_only(x)	(0 < (x)->connInfo.accessible_only)
 #define CC_default_is_c(x)	(CC_is_in_ansi_app(x) || x->ms_jet /* not only */ || TRUE /* but for any other ? */)
-/*	for CC_DSN_info */
-#define CONN_DONT_OVERWRITE		0
-#define CONN_OVERWRITE			1
 
 #ifdef	_HANDLE_ENLIST_IN_DTC_
 enum {
@@ -436,12 +387,6 @@ enum {
 
 /*	prototypes */
 ConnectionClass *CC_Constructor(void);
-enum { /* CC_conninfo_init option */
-	CLEANUP_FOR_REUSE	= 1L		/* reuse the info */
-	,COPY_GLOBALS		= (1L << 1) /* copy globals to drivers */
-};
-void		CC_conninfo_init(ConnInfo *conninfo, UInt4 option);
-void		CC_copy_conninfo(ConnInfo *to, const ConnInfo *from);
 char		CC_Destructor(ConnectionClass *self);
 int		CC_cursor_count(ConnectionClass *self);
 char		CC_cleanup(ConnectionClass *self, BOOL keepCommunication);
@@ -463,7 +408,7 @@ QResultClass *CC_send_query_append(ConnectionClass *self, const char *query, Que
 #define CC_send_query(self, query, qi, flag, stmt) CC_send_query_append(self, query, qi, flag, stmt, NULL)
 void		handle_pgres_error(ConnectionClass *self, const PGresult *pgres,
 				   const char *comment,
-				   QResultClass *res, BOOL fatal);
+				   QResultClass *res, BOOL error_not_a_notice);
 void		CC_clear_error(ConnectionClass *self);
 int		CC_send_function(ConnectionClass *conn, const char *fn_name, void *result_buf, int *actual_result_len, int result_is_int, LO_ARG *argv, int nargs);
 char		CC_send_settings(ConnectionClass *self, const char *set_query);
@@ -483,9 +428,20 @@ char	CC_get_escape(const ConnectionClass *self);
 
 const		char *CurrCat(const ConnectionClass *self);
 const		char *CurrCatString(const ConnectionClass *self);
+SQLUINTEGER	CC_get_isolation(ConnectionClass *self);
+
+SQLCHAR	*make_lstring_ifneeded(ConnectionClass *, const SQLCHAR *s, ssize_t len, BOOL);
+char	*schema_strcat(char *buf, const char *fmt, const SQLCHAR *s, SQLLEN len,
+		const SQLCHAR *, SQLLEN, ConnectionClass *conn);
+char	*schema_strcat1(char *buf, const char *fmt, const char *s1,
+				const char *s,
+				const SQLCHAR *, int, ConnectionClass *conn);
 
 void	CC_examine_global_transaction(ConnectionClass *self);
 
+
+BOOL CC_set_transact(ConnectionClass *self, UInt4 isolation);
+
 /* CC_send_query options */
 enum {
 	IGNORE_ABORT_ON_CONN	= 1L /* not set the error result even when  */
