@@ -913,7 +913,7 @@ RETCODE		SQL_API
 SQLExtendedFetch(HSTMT hstmt,
 				 SQLUSMALLINT fFetchType,
 				 SQLLEN irow,
-#if defined(WITH_UNIXODBC) && (SIZEOF_LONG != 8)
+#if defined(WITH_UNIXODBC) && (SIZEOF_LONG_INT != 8)
 				 SQLROWSETSIZE *pcrow,
 #else
 				 SQLULEN *pcrow,
