@@ -37,6 +37,8 @@
 #else  /* _DEBUG */
 #include <stdlib.h>
 #endif /* _DEBUG */
+#else  /* WIN32 */
+#include <stdlib.h>
 #endif /* WIN32 */
 
 #ifdef	_MEMORY_DEBUG_
@@ -123,6 +125,7 @@ extern "C" {
 #define UInt4 unsigned int
 #define Int2 short
 #define UInt2 unsigned short
+typedef SQLBIGINT Int8;
 typedef	UInt4	OID;
 
 #ifndef SQL_TRUE
@@ -136,58 +139,66 @@ typedef	UInt4	OID;
 #ifndef	SSIZE_T_DEFINED
 #define	ssize_t	SSIZE_T
 #define	SSIZE_T_DEFINED
-#endif
+#endif /* SSIZE_T */
 #define	FORMAT_SIZE_T	"%Iu"	/* size_t */
 #define	FORMAT_SSIZE_T	"%Id"	/* ssize_t */
 #define	FORMAT_INTEGER	"%ld"	/* SQLINTEGER */
 #define	FORMAT_UINTEGER	"%lu"	/* SQLUINTEGER */
+#define	FORMATI64	"%I64d"	/* SQLBIGINT */
+#define	FORMATI64U	"%I64u"	/* SQLUBIGINT */
 #ifdef	_WIN64
 #define	FORMAT_LEN	"%I64d" /* SQLLEN */
 #define	FORMAT_ULEN	"%I64u" /* SQLULEN */
-#define	FORMAT_LPTR	"%I64d" /* LONG_PTR */
-#define	FORMAT_ULPTR	"%I64u" /* ULONG_PTR */
-#else
+#else /* _WIN64 */
 #define	FORMAT_LEN	"%ld"	/* SQLLEN */
 #define	FORMAT_ULEN	"%lu"	/* SQLULEN */
-#define	FORMAT_LPTR	"%ld"	/* LONG_PTR */
-#define	FORMAT_ULPTR	"%lu"	/* ULONG_PTR */
 #endif /* _WIN64 */
-#else
+#else /* WIN32 */
 #define	FORMAT_SIZE_T	"%zu"	/* size_t */
 #define	FORMAT_SSIZE_T	"%zd"	/* ssize_t */
 #ifndef	HAVE_SSIZE_T
 typedef	long	ssize_t
 #endif /* HAVE_SSIZE_T */
 
-#if (SIZEOF_VOID_P == SIZEOF_LONG)
+#if (SIZEOF_VOID_P == SIZEOF_LONG) /* ILP32 or LP64 */
 typedef	long 	LONG_PTR;
 typedef	unsigned long 	ULONG_PTR;
-#define	FORMAT_LPTR	"%ld"	/* LONG_PTR */
-#define	FORMAT_ULPTR	"%lu"	/* ULONG_PTR */
-#elif defined (HAVE_LONG_LONG)
+#elif defined (HAVE_LONG_LONG) /* LLP64 */
 typedef	long long LONG_PTR;
 typedef	unsigned long long ULONG_PTR;
-#define	FORMAT_LPTR	"%lld"	/* LONG_PTR */
-#define	FORMAT_ULPTR	"%llu"	/* ULONG_PTR */
-#else
+#else /* SIZEOF_VOID_P */
 #error appropriate long pointer type not found
 #endif /* SIZEOF_VOID_P */
-#if (SIZEOF_LONG == 8)
+#if (SIZEOF_LONG == 8) /* LP64 */
 #define	FORMAT_INTEGER	"%d"	/* SQLINTEGER */
 #define	FORMAT_UINTEGER	"%u"	/* SQLUINTEGER */
+#define	FORMATI64	"%ld"	/* SQLBIGINT */
+#define	FORMATI64U	"%lu"	/* SQLUBIGINT */
 #if defined(WITH_UNIXODBC) && defined(BUILD_LEGACY_64_BIT_MODE)
 #define FORMAT_LEN	"%d"	/* SQLLEN */
 #define FORMAT_ULEN	"%u"	/* SQLULEN */
-#else
+#else /* WITH_UNIXODBC */
 #define FORMAT_LEN	"%ld"	/* SQLLEN */
 #define FORMAT_ULEN	"%lu"	/* SQLULEN */
 #endif /* WITH_UNIXODBC */
-#else
-#define	FORMAT_LEN	"%ld"	/* SQLLEN */
-#define	FORMAT_ULEN	"%lu"	/* SQLULEN */
+#else /* SIZEOF_LONG */
 #define	FORMAT_INTEGER	"%ld"	/* SQLINTEGER */
 #define	FORMAT_UINTEGER	"%lu"	/* SQLUINTEGER */
+#if defined(HAVE_LONG_LONG)
+#define	FORMATI64	"%lld"	/* SQLBIGINT */
+#define	FORMATI64U	"%llu"	/* SQLUBIGINT */
+#if (SIZEOF_VOID_P == 8) /* LLP64 */
+#define	FORMAT_LEN	"%lld"	/* SQLLEN */
+#define	FORMAT_ULEN	"%llu"	/* SQLULEN */
+#else /* SIZEOF_VOID_P ILP32 */
+#define	FORMAT_LEN	"%ld"	/* SQLLEN */
+#define	FORMAT_ULEN	"%lu"	/* SQLULEN */
 #endif /* SIZEOF_VOID_P */
+#else /* HAVE_LONG_LONG */
+#define	FORMAT_LEN	"%ld"	/* SQLLEN */
+#define	FORMAT_ULEN	"%lu"	/* SQLULEN */
+#endif /* HAVE_LONG_LONG */
+#endif /* SIZEOF_LONG */
 #endif /* WIN32 */
 #define	CAST_PTR(type, ptr)	(type)((LONG_PTR)(ptr))
 #define	CAST_UPTR(type, ptr)	(type)((ULONG_PTR)(ptr))
@@ -527,7 +538,66 @@ void		logs_on_off(int cnopen, int, int);
 
 #define	LENADDR_SHIFT(x, sft)	((x) ? (SQLLEN *)((char *)(x) + (sft)) : NULL)
 
+/*	Structure to hold all the connection attributes for a specific
+	connection (used for both registry and file, DSN and DRIVER)
+*/
+typedef struct
+{
+	char		dsn[MEDIUM_REGISTRY_LEN];
+	char		desc[MEDIUM_REGISTRY_LEN];
+	char		drivername[MEDIUM_REGISTRY_LEN];
+	char		server[MEDIUM_REGISTRY_LEN];
+	char		database[MEDIUM_REGISTRY_LEN];
+	char		username[MEDIUM_REGISTRY_LEN];
+	pgNAME		password;
+	char		port[SMALL_REGISTRY_LEN];
+	char		sslmode[16];
+	char		onlyread[SMALL_REGISTRY_LEN];
+	char		fake_oid_index[SMALL_REGISTRY_LEN];
+	char		show_oid_column[SMALL_REGISTRY_LEN];
+	char		row_versioning[SMALL_REGISTRY_LEN];
+	char		show_system_tables[SMALL_REGISTRY_LEN];
+	char		translation_dll[MEDIUM_REGISTRY_LEN];
+	char		translation_option[SMALL_REGISTRY_LEN];
+	char		password_required;
+	pgNAME		conn_settings;
+	pgNAME		pqopt;
+	signed char	allow_keyset;
+	signed char	updatable_cursors;
+	signed char	lf_conversion;
+	signed char	true_is_minus1;
+	signed char	int8_as;
+	signed char	bytea_as_longvarbinary;
+	signed char	use_server_side_prepare;
+	signed char	lower_case_identifier;
+	signed char	rollback_on_error;
+	signed char	force_abbrev_connstr;
+	signed char	bde_environment;
+	signed char	fake_mss;
+	signed char	cvt_null_date_string;
+	signed char	accessible_only;
+	signed char	ignore_round_trip_time;
+	signed char	disable_keepalive;
+	signed char	gssauth_use_gssapi;
+	UInt4		extra_opts;
+	Int4		keepalive_idle;
+	Int4		keepalive_interval;
+#ifdef	_HANDLE_ENLIST_IN_DTC_
+	signed char	xa_opt;
+#endif /* _HANDLE_ENLIST_IN_DTC_ */
+	GLOBAL_VALUES drivers;		/* moved from driver's option */
+} ConnInfo;
+
+#define SUPPORT_DESCRIBE_PARAM(conninfo_) (conninfo_->use_server_side_prepare)
+
 int	initialize_global_cs(void);
+enum { /* CC_conninfo_init option */
+	CLEANUP_FOR_REUSE	= 1L		/* reuse the info */
+	,COPY_GLOBALS		= (1L << 1) /* copy globals to drivers */
+};
+void	CC_conninfo_init(ConnInfo *conninfo, UInt4 option);
+void	CC_conninfo_release(ConnInfo *conninfo);
+void	CC_copy_conninfo(ConnInfo *ci, const ConnInfo *sci);
 #ifdef	POSIX_MULTITHREAD_SUPPORT
 #if	!defined(HAVE_ECO_THREAD_LOCKS)
 #define	POSIX_THREADMUTEX_SUPPORT
